@@ -2,17 +2,17 @@
 # micropipenv
 # Copyright(C) 2020 Fridolin Pokorny
 #
-# This program is free software: you can redistribute it and / or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software: you can redistribute it and / or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """A lightweight wrapper for pip to support Pipenv files.
@@ -26,25 +26,9 @@ dependencies using raw pip. The virtual environment is not created, but one can
 issue `python3 -m venv venv/ && . venv/bin/activate` to create one.
 """
 
-__version__ = "0.1.6"
+__version__ = "0.2.0"
 __author__ = "Fridolin Pokorny <fridex.devel@gmail.com>"
 __title__ = "micropipenv"
-__all__ = [
-    "ArgumentsError",
-    "ExtrasMissing",
-    "FileNotFound",
-    "FileReadError",
-    "get_requirements_sections",
-    "HashMismatch",
-    "install",
-    "install_pipenv",
-    "install_requirements",
-    "install_poetry",
-    "main",
-    "MicropipenvException",
-    "PythonVersionMismatch",
-    "requirements",
-]
 
 import argparse
 import hashlib
@@ -55,8 +39,9 @@ import re
 import subprocess
 import sys
 import tempfile
-from collections import deque
+from collections import deque, OrderedDict
 from itertools import chain
+from pathlib import Path
 from urllib.parse import urlparse
 
 from pip._vendor.packaging.requirements import Requirement
@@ -95,6 +80,7 @@ if TYPE_CHECKING:
     from typing import Optional
     from typing import Tuple
     from typing import Union
+    from pip._internal.req.req_file import ParsedRequirement
 
 
 _LOGGER = logging.getLogger(__title__)
@@ -104,6 +90,11 @@ _PIP_BIN = os.getenv("MICROPIPENV_PIP_BIN", "pip")
 _DEBUG = int(os.getenv("MICROPIPENV_DEBUG", 0))
 _NO_LOCKFILE_PRINT = int(os.getenv("MICROPIPENV_NO_LOCKFILE_PRINT", 0))
 _NO_LOCKFILE_WRITE = int(os.getenv("MICROPIPENV_NO_LOCKFILE_WRITE", 0))
+_FILE_METHOD_MAP = OrderedDict({  # The order here defines priorities
+    "Pipfile.lock": "pipenv",
+    "poetry.lock": "poetry",
+    "requirements.txt": "requirements",
+})
 
 
 class MicropipenvException(Exception):
@@ -150,6 +141,10 @@ class PoetryError(MicropipenvException):
     """Raised when any of the pyproject.toml or poetry.lock file has any issue."""
 
 
+class CompatibilityError(MicropipenvException):
+    """Raised when internal pip API is incompatible with micropipenv."""
+
+
 def _traverse_up_find_file(file_name):  # type: (str) -> str
     """Traverse the root up, find the given file by name and return its path."""
     path = os.getcwd()
@@ -274,7 +269,7 @@ def install_pipenv(
                 "Pipfile {!r}, aborting deployment".format(pipfile_lock_hash, pipfile_hash)
             )
 
-    tmp_file = tempfile.NamedTemporaryFile("w", prefix="requirements.txt", delete=False)
+    tmp_file = tempfile.NamedTemporaryFile("w", prefix="requirements_micropipenv-", suffix=".txt", delete=False)
     _LOGGER.debug("Using temporary file for storing requirements: %r", tmp_file.name)
 
     cmd = [pip_bin, "install", "--no-deps", "--disable-pip-version-check", "-r", tmp_file.name, *(pip_args or [])]
@@ -346,6 +341,110 @@ def _instantiate_package_finder(pip_session):  # type: (PipSession) -> PackageFi
         return PackageFinder.create(link_collector=link_collector, selection_prefs=selection_prefs,)
 
 
+def _get_requirement_info(requirement):  # type: (ParsedRequirement) -> Dict[str, Any]
+    """Get information about the requirement supporting multiple pip versions.
+
+    This function acts like a compatibility layer across multiple pip releases supporting
+    changes in pip's internal API to obtain information about requirements.
+    """
+    # Editable requirements.
+    editable = getattr(requirement, "editable", False) or getattr(requirement, "is_editable", False)
+
+    is_url = False
+    req = None
+    if hasattr(requirement, "req"):
+        req = requirement.req
+    elif hasattr(requirement, "requirement") and not editable:
+        if not requirement.requirement.startswith("git+"):
+            req = Requirement(requirement.requirement)
+        else:
+            is_url = True
+
+    # Link for requirements passed by URL or using path.
+    link = None
+    if editable:
+        if hasattr(requirement, "link"):
+            link = str(requirement.link)
+        elif req is not None:
+            link = req.url
+        elif hasattr(requirement, "requirement"):
+            link = str(requirement.requirement)
+        else:
+            raise CompatibilityError
+
+    # Requirement name.
+    if editable:
+        name = str(link)
+    elif hasattr(requirement, "name"):
+        name = requirement.name
+    elif req is not None:
+        name = req.name
+    elif hasattr(requirement, "requirement") and is_url:
+        name = requirement.requirement
+    else:
+        raise CompatibilityError
+
+    # Version specifier.
+    version_specifier = None
+    version_specifier_length = None
+    if not editable and not is_url:
+        if hasattr(requirement, "specifier"):
+            version_specifier = str(requirement.specifier)
+            version_specifier_length = len(requirement.specifier)
+        elif req is not None:
+            # pip>=20
+            version_specifier = str(req.specifier)
+            version_specifier_length = len(req.specifier)
+        else:
+            raise CompatibilityError
+
+    # Artifact hashes.
+    hash_options = None
+    if not editable and not is_url:
+        if hasattr(requirement, "options"):
+            hash_options = requirement.options.get("hashes")
+        else:
+            hash_options = requirement.hash_options  # More recent pip.
+
+    hashes = {}
+    for hash_type, hashes_present in hash_options.items() if hash_options else []:
+        hashes = {
+            "hash_type": hash_type,
+            "hashes_present": hashes_present,
+        }
+
+    # Markers.
+    markers = None
+    if not editable and not is_url:
+        if hasattr(requirement, "markers"):
+            markers = requirement.markers
+        elif req is not None:
+            markers = req.marker
+        else:
+            raise CompatibilityError
+
+    # Extras.
+    extras = None
+    if not editable and not is_url:
+        if hasattr(requirement, "extras"):
+            extras = requirement.extras
+        elif req is not None:
+            extras = req.extras
+        else:
+            raise CompatibilityError
+
+    return {
+        "editable": editable,
+        "version_specifier": version_specifier,
+        "version_specifier_length": version_specifier_length,
+        "hashes": hashes,
+        "link": link,
+        "extras": extras,
+        "markers": markers,
+        "name": name,
+    }
+
+
 def _requirements2pipfile_lock():  # type: () -> Dict[str, Any]
     """Parse requirements.txt file and return its Pipfile.lock representation."""
     requirements_txt_path = _traverse_up_find_file("requirements.txt")
@@ -355,43 +454,39 @@ def _requirements2pipfile_lock():  # type: () -> Dict[str, Any]
 
     result = {}  # type: Dict[str, Any]
     for requirement in parse_requirements(filename=requirements_txt_path, session=PipSession(), finder=finder):
+        requirement_info = _get_requirement_info(requirement)
         entry = {}  # type: Dict[str, Any]
 
-        if not requirement.editable:
-            version = str(requirement.specifier)
-
-            if requirement.specifier is None or not (
-                requirement.has_hash_options and len(requirement.specifier) == 1 and version.startswith("==")
+        if not requirement_info["editable"]:
+            if requirement_info["version_specifier"] is None or not (
+                requirement_info["hashes"]
+                and requirement_info["version_specifier_length"] == 1
+                and requirement_info["version_specifier"].startswith("==")
             ):
                 # Not pinned down software stack using pip-tools.
                 raise PipRequirementsNotLocked
 
-            if hasattr(requirement, "options"):
-                hash_options = requirement.options.get("hashes")
-            else:
-                hash_options = requirement.hash_options  # More recent pip.
-
             hashes = []
-            for hash_type, hashes_present in hash_options.items():
-                hashes.extend(["{}:{}".format(hash_type, h) for h in hashes_present])
+            for hash_ in requirement_info["hashes"]["hashes_present"]:
+                hashes.append("{}:{}".format(requirement_info["hashes"]["hash_type"], hash_))
 
             entry["hashes"] = sorted(hashes)
-            entry["version"] = version
+            entry["version"] = requirement_info["version_specifier"]
         else:
             entry["editable"] = True
-            entry["path"] = str(requirement.link)
+            entry["path"] = requirement_info["link"]
 
-        if requirement.extras:
-            entry["extras"] = sorted(requirement.extras)
+        if requirement_info["extras"]:
+            entry["extras"] = sorted(requirement_info["extras"])
 
-        if requirement.markers:
-            entry["markers"] = str(requirement.markers)
+        if requirement_info["markers"]:
+            entry["markers"] = str(requirement_info["markers"])
 
         if entry.get("editable", False):
             # Create a unique name for editable to avoid possible clashes.
             requirement_name = hashlib.sha256(json.dumps(entry, sort_keys=True).encode("utf8")).hexdigest()
         else:
-            requirement_name = requirement.name
+            requirement_name = requirement_info["name"]
 
         # We add all dependencies to default, develop should not be present in requirements.txt file, but rather
         # in dev-requirements.txt or similar.
@@ -639,24 +734,26 @@ def install(
 ):  # type: (Optional[str], bool, bool, Optional[List[str]]) -> None
     """Perform installation of requirements based on the method used."""
     if method is None:
-        try:
-            _traverse_up_find_file("Pipfile.lock")
-            method = "pipenv"
-        except FileNotFound as exc:
+        paths = []
+        for file_name in _FILE_METHOD_MAP.keys():
             try:
-                _LOGGER.info("Failed to find Pipfile.lock, looking up poetry.lock: %s", str(exc))
-                _traverse_up_find_file("poetry.lock")
-                method = "poetry"
-            except FileNotFound as exc:
-                try:
-                    _LOGGER.info("Failed to find Poetry lock file, looking up requirements.txt: %s", str(exc))
-                    _traverse_up_find_file("requirements.txt")
-                    method = "requirements"
-                except FileNotFound as exc:
-                    raise FileNotFound(
-                        "Failed to find Pipfile.lock, poetry.lock or requirements.txt "
-                        "in the current directory or any of its parent: {}".format(os.getcwd())
-                    ) from exc
+                paths.append(Path(_traverse_up_find_file(file_name)))
+            except FileNotFound:
+                pass
+
+        if not paths:
+            raise FileNotFound(
+                "Failed to find Pipfile.lock, poetry.lock or requirements.txt "
+                "in the current directory or any of its parent: {}".format(os.getcwd()))
+
+        _LOGGER.debug("Dependencies definitions found: %s", paths)
+        # The longest path means that we are as close to CWD as possible.
+        # Sorting is also stable which means that two paths with the same
+        # lenght will have the same order when sorted which keeps priorities
+        # from _FILE_METHOS_MAP.
+        longest_path = sorted(paths, key=lambda p: len(p.parts), reverse=True)[0]
+        _LOGGER.debug("Choosen definition: %s", str(longest_path))
+        method = _FILE_METHOD_MAP[longest_path.name]
 
     if method == "requirements":
         if deploy:
